/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdio.h>
#include <zephyr/device.h>
#include <zephyr/drivers/pinctrl.h>
#include <zephyr/pm/device.h>

#define UART0_NODE DT_NODELABEL(uart0)

static const struct device *uart0 = DEVICE_DT_GET(UART0_NODE);

PINCTRL_DT_DEV_CONFIG_DECLARE(UART0_NODE);

struct pinctrl_dev_config *uart0_config = PINCTRL_DT_DEV_CONFIG_GET(UART0_NODE);

/* UART0 alternative configurations (default and sleep states) */
PINCTRL_DT_STATE_PINS_DEFINE(DT_PATH(zephyr_user), uart0_alt_default);
PINCTRL_DT_STATE_PINS_DEFINE(DT_PATH(zephyr_user), uart0_alt_sleep);
/* UART0 original configurations (default and sleep states) */
PINCTRL_DT_STATE_PINS_DEFINE(DT_PATH(zephyr_user), uart0_default);
PINCTRL_DT_STATE_PINS_DEFINE(DT_PATH(zephyr_user), uart0_sleep);


static const struct pinctrl_state uart0_alt[] = {
	PINCTRL_DT_STATE_INIT(uart0_alt_default, PINCTRL_STATE_DEFAULT),
	PINCTRL_DT_STATE_INIT(uart0_alt_sleep, PINCTRL_STATE_SLEEP),
};

static const struct pinctrl_state uart0_original[] = {
	PINCTRL_DT_STATE_INIT(uart0_default, PINCTRL_STATE_DEFAULT),
	PINCTRL_DT_STATE_INIT(uart0_sleep, PINCTRL_STATE_SLEEP),
};

void uart_pinout_update(void) 
{
	int err;
	static bool alternate_pinout;
	
	/* Temporalily disable the UART to allow for pinout to be updated. The current UART pins will be 
	 * reconfigured back to their default state (input, disconnected) to prevent any leakage.
	 * 
	 * UART pins should not be changed while the peripheral is enabled. Ref. 
	 * https://infocenter.nordicsemi.com/topic/ps_nrf52840/uarte.html#concept_wmv_f2m_wr
	 */
	err = pm_device_action_run(uart0, PM_DEVICE_ACTION_SUSPEND);
	if (err) {
		return;
	}
	
	/* Toggle between the alternate pinout and the original for this test */
	alternate_pinout = !alternate_pinout;

	if (alternate_pinout) {
		err = pinctrl_update_states(uart0_config, uart0_alt, ARRAY_SIZE(uart0_alt));
		if (err) {
			return;
		}
	}
	else {
		err = pinctrl_update_states(uart0_config, uart0_original, ARRAY_SIZE(uart0_original));
		if (err) {
			return;
		}
	}
	/* Re-enable the UART with the new pinout */
	err = pm_device_action_run(uart0, PM_DEVICE_ACTION_RESUME);
	if (err) {
		return;
	}
		
}

int main(void)
{
	//memcpy()

	while(1) {
		printf("Hello World! %s\n", CONFIG_BOARD);
		k_msleep(1000);
		uart_pinout_update();
	}
}
